#!/bin/bash

### The postprocessing part of the neural network. This is run at each generation, after all individual members are
### completed.
### 1. Loops the directories of each member and collects the cost value. Appends this to a copy of 00.csv and saves as a
###    new file, so that there will ultimately be a .csv file of parameters and their associated costs at every
###    generation
### 2. Performs a genetic algorithm (Biased Random Key Genetic Algorithm (http://mauricio.resende.info/doc/brkgaAPI.pdf)
###    the set of parameters that are generated are saved onto 00.csv, to be used in the next generation.


### 1. Loops the directories of each member and collects the cost value. Appends this to a copy of 00.csv and saves as a
###    new file, so that there will ultimately be a .csv file of parameters and their associated costs at every
###    generation

INPUT=/gpfs/scratch/bsc32/bsc32786/%EXPID%/EXP00/

module load python

function makecsv {

parameters="$(python - $1 << END

import numpy as np
import os
import pandas

dir='$1'
print(dir)
os.chdir(dir)

#parameters=pandas.read_csv('00.csv')
#print(parameters)
#parameters[3]=np.nan

members=os.listdir(os.getcwd())
members=[i for i in members if i[:2]=='fc' if len(i)==4]

os.chdir(dir+'parameters')
parameters=np.genfromtxt('00.csv')
parameters=np.append(parameters,np.zeros((np.shape(parameters)[0],1)),axis=1)

for member in members:
    ind=int(member[2:])
    os.chdir(dir+'/'+member)
    f=open('cost.txt','r')
    cost=float(f.read())
    f.close()
    print(cost)
    parameters[ind,3]=cost
    #parameters[3][ind]=cost
print(parameters)

os.chdir(dir+'/parameters')
gen=len(os.listdir(dir+'/parameters'))
print(gen)
gen="{0:0=2d}".format(gen)+'.csv'

np.savetxt(gen,parameters)
#parameters.to_csv(gen)

END
)"
}

makecsv $INPUT


### 2. Performs a genetic algorithm (Biased Random Key Genetic Algorithm (http://mauricio.resende.info/doc/brkgaAPI.pdf)
###    the set of parameters that are generated are saved onto 00.csv, to be used in the next generation.

SCRHOME=/gpfs/scratch/bsc32/bsc32786/
MODEL=%EXPID%/EXP00/parameters
cd $SCRHOME$MODEL

module load python

function GA {

GA="$(python - $1 << END

import numpy as np
import random
import pandas
import os

def geneticalgorithm(input):

# Biased Random Key Genetic Algorithm (BRKGA)
# One generation crossover/mutation
# input - dataframe ((P+1)*M), extra column for values of the cost function
# output - dataframe (P*M) of new parameters

# P = number of parameters
# N = number of members

    #(N,P) = input.shape
    (N,P) = np.shape(input)
    P=P-1

    E=0.15 # proportion of elite
    M=0.1  # proportion of mutants
    rho=0.7 # probability of each crossover being assigned to the elite member

    NE=int(N*E)+1 # number of elite members (rounded up)
    NM=int(N*M)+1 # number of mutant members (rounded up)
    NC=N-(NE+NM) # number of crossover member

    # sorting the input by cost and selecting elite and non-elite members
    input=pandas.DataFrame(input)
    input=input.sort_values(input.shape[1]-1).values
    inputE=input[:NE,:]
    inputnE=input[NE:,:]

    # putting elite members into output
    output=input[:NE,:-1]

    # generating mutants and putting into output
    for i in range(NM):
        r=np.zeros((1,P))
        r[0,0]=random.random()*0.035+0.015
        r[0,1]=random.random()*2.+1.
        r[0,2]=random.random()*3000.+1500.
        output=np.append(output,r,axis=0)

         # crossover
    for i in range(NC):
        r=np.zeros((1,P))
        for j in range(P):
            en=inputE[np.random.randint(inputE.shape[0], size=1), :]
            nen=inputnE[np.random.randint(inputnE.shape[0], size=1), :]
            a=random.random()
            if a>rho:
                r[0,j]=nen[0,j]
            else:

                r[0,j]=en[0,j]
        output=np.append(output,r,axis=0)

    return output

os.chdir('$1')
gen=len(os.listdir('$1'))-1
gen="{0:0=2d}".format(gen)+'.csv'

A=np.genfromtxt(gen)

B=geneticalgorithm(A)
if os.path.exists("00.csv"):
    os.remove("00.csv")
np.savetxt('00.csv',B)

END
)"
}

GA $SCRHOME$MODEL